﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.ServiceModel.Description;
using System.IO;

namespace TestServiceApp
{
	[ServiceContract]
	interface ITestService
	{
		[OperationContract]
		[WebGet(UriTemplate = "{category}/TestMethod1.svc?value={value}")]
		string TestMethod1(string category, string value);

		[OperationContract]
		[WebInvoke(UriTemplate = "TestService.svc")]
		string TestMethod2(string arg);
	}

	class TestService : ITestService
	{
		public string TestMethod1(string category, string value)
		{
			Console.WriteLine("TestMethod1 invoked... category={0}, value={1}",
				category, value);
			return String.Format("category={0}, value={1}",
				category, value);
		}

		public string TestMethod2(string arg)
		{
			Console.WriteLine("TestMethod2 invoked... arg={0}", arg);
			return String.Format("arg={0}", arg);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Advanced Web Programming Model Test Service...");

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/web/advanced"));
			BasicHttpBinding binding1 = new BasicHttpBinding();
			binding1.TransferMode = TransferMode.Streamed;
			host.AddServiceEndpoint(typeof(ITestService), binding1, "soap");

			WebHttpBinding binding2 = new WebHttpBinding();
			ServiceEndpoint ep = host.AddServiceEndpoint(typeof(ITestService), binding2, "web");
			ep.Behaviors.Add(new WebHttpBehavior());

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
