﻿using System;
using System.ServiceModel;
using System.Transactions;
using System.ServiceModel.Channels;

namespace TxServiceApp
{
	[ServiceContract]
	interface ITxFlowService
	{
		[OperationContract]
		[TransactionFlow(TransactionFlowOption.Mandatory)]
		void TxMethod1();

		[OperationContract]
		[TransactionFlow(TransactionFlowOption.Allowed)]
		void TxMethod2();

		[OperationContract]
		[TransactionFlow(TransactionFlowOption.NotAllowed)]
		void TxMethod3();
	}

	class TxFlowService : ITxFlowService
	{
		[OperationBehavior(TransactionScopeRequired = true)]
		public void TxMethod1()
		{
			Console.WriteLine("TxMethod1 invoked...");
			ShowCurrentTransactionInfo();
		}

		[OperationBehavior(TransactionScopeRequired = true)]
		public void TxMethod2()
		{
			Console.WriteLine("TxMethod2 invoked...");
			ShowCurrentTransactionInfo();
		}

		[OperationBehavior(TransactionScopeRequired = true)]
		public void TxMethod3()
		{
			Console.WriteLine("TxMethod3 invoked...");
			ShowCurrentTransactionInfo();
		}

		#region "Tx Info Helper"

		static string GetTransactionId(Transaction tx)
		{
			if (tx.TransactionInformation.DistributedIdentifier != Guid.Empty) {
				return tx.TransactionInformation.DistributedIdentifier.ToString();
			}
			else {
				return tx.TransactionInformation.LocalIdentifier;
			}
		}

		static void ShowCurrentTransactionInfo()
		{
			bool isInTx = (Transaction.Current != null);

			Console.WriteLine("  >> Transaction Information:");
			Console.WriteLine("\tIs in transaction : {0}", isInTx);
			if (isInTx == true) {
				Transaction tx = Transaction.Current;
				TransactionInformation ti = tx.TransactionInformation;
				bool isDtx = (tx.TransactionInformation.DistributedIdentifier != Guid.Empty);
				string txId = GetTransactionId(tx);
				DateTime creationTime = ti.CreationTime;

				Console.WriteLine("\tTransaction type : {0}", isDtx ? "Distributed" : "Local");
				Console.WriteLine("\tTransaction id : {0}", txId);
				Console.WriteLine("\tCreation time : {0}", creationTime);
			}
		}

		#endregion
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Transaction Flow Test Service......");

			ServiceHost host = new ServiceHost(typeof(TxFlowService));

			//CustomBinding binding = new CustomBinding();
			//binding.Elements.Add(new TransactionFlowBindingElement());
			//binding.Elements.Add(new BinaryMessageEncodingBindingElement());
			//binding.Elements.Add(new TcpTransportBindingElement());
			//host.Description.Endpoints[1].Binding = binding;

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
