﻿using System;
using System.Transactions;
using System.Data;
using System.Data.SqlClient;

namespace UsingRawTransaction
{
	class Program
	{
		const string ConnStr = "SERVER=(local);UID=Tester;PWD=test";

		static void Main(string[] args)
		{
			Console.WriteLine("Transaction processing using raw Transaction object......");

			//SimpleUnconditionalTransaction();

			//using (TransactionScope scope = new TransactionScope()) {
			//    SafeNewTransaction();
			//    // Complete 호출을 수행하지 않았지만 SafeNewTransaciton 의 작업 사항은
			//    // 커밋된다.
			//}

			// Complete 메쏘드 호출에 따라 트랜잭션은 커밋되거나 롤백된다.
			using (TransactionScope scope = new TransactionScope()) {
				EmulateRequiredTransaction();
				//scope.Complete();
			}
		}

		static void SimpleUnconditionalTransaction()
		{
			Console.WriteLine("Simple Transaction Test with Transaction Option");

			string query = "INSERT INTO TestTable VALUES(99, 'TempData'); " +
						"SELECT @@IDENTITY";
			TransactionOptions options = new TransactionOptions();
			options.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;
			options.Timeout = TimeSpan.FromSeconds(120);

			CommittableTransaction txCommittable = new CommittableTransaction(options);
			Transaction.Current = txCommittable;

			try {
				SqlConnection conn = new SqlConnection(ConnStr);
				SqlCommand cmd = new SqlCommand(query, conn);

				conn.Open();
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				conn.Close();
				txCommittable.Commit();
			}
			catch (Exception) {
				txCommittable.Rollback();
				throw;
			}
			finally {
				Transaction.Current = null;
				txCommittable.Dispose();
			}
			Console.WriteLine("   Done...");
		}

		static void SafeNewTransaction()
		{
			Console.WriteLine("Simple Transaction Test with Transaction Option");

			string query = "INSERT INTO TestTable VALUES(99, 'TempData'); " +
						"SELECT @@IDENTITY";
			TransactionOptions options = new TransactionOptions();
			options.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;
			options.Timeout = TimeSpan.FromSeconds(120);

			CommittableTransaction txCommittable = new CommittableTransaction(options);
			Transaction oldAmbientTx = Transaction.Current;
			Transaction.Current = txCommittable;

			try {
				SqlConnection conn = new SqlConnection(ConnStr);
				SqlCommand cmd = new SqlCommand(query, conn);

				conn.Open();
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				conn.Close();
				txCommittable.Commit();
			}
			catch (Exception) {
				txCommittable.Rollback();
				throw;
			}
			finally {
				Transaction.Current = oldAmbientTx;
				txCommittable.Dispose();
			}
			Console.WriteLine("   Done...");
		}

		static void EmulateRequiredTransaction()
		{
			Console.WriteLine("Simple Transaction Test with Transaction Option");

			string query = "INSERT INTO TestTable VALUES(99, 'TempData'); " +
						"SELECT @@IDENTITY";
			TransactionOptions options = new TransactionOptions();
			options.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;
			options.Timeout = TimeSpan.FromSeconds(120);

			Transaction oldAmbientTx = Transaction.Current;
			Transaction newTx;
			if (oldAmbientTx == null) {
				newTx = new CommittableTransaction();
			}
			else {
				newTx = oldAmbientTx.DependentClone(DependentCloneOption.RollbackIfNotComplete);
			}
			Transaction.Current = newTx;

			try {
				SqlConnection conn = new SqlConnection(ConnStr);
				SqlCommand cmd = new SqlCommand(query, conn);

				conn.Open();
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				conn.Close();
				if (newTx is CommittableTransaction) {
					((CommittableTransaction)newTx).Commit();
				}
				else {
					((DependentTransaction)newTx).Complete();
				}
			}
			catch (Exception) {
				newTx.Rollback();
				throw;
			}
			finally {
				Transaction.Current = oldAmbientTx;
				newTx.Dispose();
			}
			Console.WriteLine("   Done...");
		}
	}
}
