﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Messaging;

namespace MsmqServiceApp
{
	[ServiceContract]
	[DeliveryRequirements(QueuedDeliveryRequirements = QueuedDeliveryRequirementsMode.Required)]
	interface IMsmqService
	{
		[OperationContract(IsOneWay = true)]
		void TestMethod(string arg);
	}

	class MsmqService : IMsmqService
	{
		public void TestMethod(string arg)
		{
			Console.WriteLine("TestMethod invoked... arg={0}", arg);
			ReportJournalContent();
		}

		public static void ReportJournalContent()
		{
			string queuePath = ".\\private$\\GenericTxQueue;journal";
			MessageQueue queue = new MessageQueue(queuePath);
			Console.WriteLine("Inspecting journal queue : {0}", queue.Path);

			// GetAllMessages는 메시지를 큐에서 제거하지 않는다(PEEK)
			Message[] msgs = queue.GetAllMessages();
			Console.WriteLine("  Message count : {0}", msgs.Length);
			for(int i=0; i < msgs.Length; i++) {
				Console.WriteLine("   Msg #{0} : Id={1}", i, msgs[i].Id);
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Journaling Test Service Sample......");

			string queuePath = ".\\private$\\GenericTxQueue";
			if (MessageQueue.Exists(queuePath) == false) {
				MessageQueue.Create(queuePath);
			}
			// 저널 큐 사용 설정.
			// 동일 컴퓨터 상에서 클라이언트가 저널링을 사용하고 메시지 큐도
			// 저널링을 사용하면 클라이언트의 저널링은 사용되지 않는다.
			// 클라이언트 측 저널링 테스트를 위해서 다음 코드를 사용하지 않았다.
			// 클라이언트와 서비스가 다르다면 다음 코드의 주석을 해제할 수도 있다.
			//MessageQueue queue = new MessageQueue(queuePath);
			//if (queue.UseJournalQueue == false) {
			//    queue.UseJournalQueue = true;
			//}
			// 저널 큐의 내용 표시
			MsmqService.ReportJournalContent();

			ServiceHost host = new ServiceHost(typeof(MsmqService));
			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
