﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.ServiceModel;
using System.ServiceModel.Description;
using ReliableClientApp.ReliableService;

namespace ReliableClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Ordered Delivery Client Sample......");

			// 이 예제는 BasicReliableSessionSample에서 사용했던 SimpleHttpRouter를
			// 사용한다. 따라서 이 클라이언트를 수행하기 전에 SimpleHttpRouter를 먼저
			// 수행해야 한다.
			Uri intermediateUri = new Uri("http://loner:8080/intermediate/httprouter");
			ClientViaBehavior behavior = new ClientViaBehavior(intermediateUri);
			using (ReliableMessagingClient proxy = new ReliableMessagingClient()) {
				proxy.Endpoint.Behaviors.Add(behavior);
				// 서비스와 클라이언트의 Ordered 속성을 true 혹은 false로 바꾸어서
				// 테스트 함으로써 순차 전송을 확인한다.
				// Ordered 속성이 false 이면 SimpleHttpRouter가 메시지를 유실함에 따라
				// 메시지가 순차적으로 서비스에 도착하지 않게 된다.
				// 하지만 Ordered 속성이 true 이면 메시지가 유실되더라도 클라이언트에서
				// 호출한 순서대로 서비스에 메시지가 도착할 것이다.
				WSHttpBinding binding = (WSHttpBinding)proxy.Endpoint.Binding;
				binding.ReliableSession.Ordered = true;

				int completionCount = 0;
				int concurrentCallCount = 10;
				IAsyncResult[] asyncResults = new IAsyncResult[concurrentCallCount];
				AsyncCallback completionCallback = delegate(IAsyncResult ar) {
					try {
						proxy.EndReliableMethod(ar);
					}
					catch (Exception ex) {
						Console.WriteLine("Error : {0}", ex.Message);
					}
					Interlocked.Increment(ref completionCount);
				};
				for (int i = 0; i < concurrentCallCount; i++) {
					string msg = String.Format("Message #{0}", i);
					asyncResults[i] = proxy.BeginReliableMethod(msg, completionCallback, null);
					Console.WriteLine("Invoke : {0}", msg);
				}

				while (completionCount < concurrentCallCount) {
					Thread.Sleep(50);
					if (Console.KeyAvailable) {
						break;
					}
				}
			}
		}
	}
}
