﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;

namespace ReliableServiceApp
{
	[ServiceContract]
	interface IReliableMessaging
	{
		[OperationContract]
		void ReliableMethod(string arg);
	}

	class ReliableMessagingService : IReliableMessaging
	{
		public void ReliableMethod(string arg)
		{
			Console.WriteLine("Reliable Method Invoked... arg={0}", arg);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Ordered Delivery Test Service ......");

			// 커스텀 바인딩 예제 코드
			// 서비스가 커스텀 바인딩을 사용하더라도 이 바인딩은 신뢰할 수 있는 세션이
			// enable 된 WSHttpBinding과 동등하다. 따라서 svcutil.exe를 통해 클라이언트 측
			// 프록시 코드를 생성하면 마치 서비스가 WSHttpBinding을 사용한 것 처럼
			// 코드와 app.config 가 생성될 것이다.
			CustomBinding binding = new CustomBinding();
			HttpTransportBindingElement transportElement = new HttpTransportBindingElement();
			TextMessageEncodingBindingElement encodingElement = new TextMessageEncodingBindingElement();
			ReliableSessionBindingElement reliableElement = new ReliableSessionBindingElement();
			// 이 속성의 값을 false 로 바꾸고(클라이언트 역시 false로 바꾸어야 한다)
			// 테스트 하면 메시지가 클라이언트에서 호출한 순서대로 도착하지 않는다.
			reliableElement.Ordered = true;
			reliableElement.MaxTransferWindowSize = 32;
			binding.Elements.Add(reliableElement);
			binding.Elements.Add(encodingElement);
			binding.Elements.Add(transportElement);

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;

			ServiceHost host = new ServiceHost(typeof(ReliableMessagingService),
				new Uri("http://localhost/wcf/example/rm/basic"));
			host.AddServiceEndpoint(typeof(IReliableMessaging), binding, "");
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Service started. Press any key to stop......");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
