﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Principal;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
	}

	class SecureService : ISecureService
	{
		[OperationBehavior(Impersonation = ImpersonationOption.NotAllowed)]
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = OperationContext.Current.ServiceSecurityContext;
			if (ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ctx.PrimaryIdentity.Name);
			}
			WindowsIdentity currentIdentity = WindowsIdentity.GetCurrent();
			Console.WriteLine("  Current thread account = {0}", currentIdentity.Name);

			ctx.WindowsIdentity.Impersonate();
			WindowsIdentity currentIdentity2 = WindowsIdentity.GetCurrent();
			Console.WriteLine("  Current thread account = {0}", currentIdentity2.Name);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Test Service of TransportCredentialOnly...");

			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.TransportCredentialOnly;
			binding.Security.Transport.ClientCredentialType =
				HttpClientCredentialType.Windows;

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("http://localhost/wcf/example/security/transportcredentialonly"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Description.Behaviors.Add(behavior);
			host.Open();
			Console.WriteLine("The Service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
