﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;

namespace TestServiceApp
{
	[ServiceContract]
	interface ITestService
	{
		[OperationContract]
		void TestMethod();
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class TestService : ITestService
	{
		public void TestMethod()
		{
			Console.WriteLine("TestMethod invoked......");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Basic CustomBinding Test Service......");

			HttpTransportBindingElement transportElement =
				new HttpTransportBindingElement();
			MtomMessageEncodingBindingElement encodingElement =
				new MtomMessageEncodingBindingElement();
			ReliableSessionBindingElement reliableSessionElement =
				new ReliableSessionBindingElement();
			reliableSessionElement.Ordered = true;
			CustomBinding binding = new CustomBinding();
			binding.Elements.Add(reliableSessionElement);
			binding.Elements.Add(encodingElement);
			binding.Elements.Add(transportElement);

			//WSHttpBinding binding = new WSHttpBinding();
			//binding.ReliableSession.Enabled = true;
			//binding.ReliableSession.Ordered = true;
			//binding.MessageEncoding = WSMessageEncoding.Mtom;
			//binding.Security.Mode = SecurityMode.None;

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/custombinding/basic"));
			host.AddServiceEndpoint(typeof(ITestService), binding, "");

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
