﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Configuration;

namespace BindingExtension
{
	public class CustomHttpBinding : Binding
	{
		private SecurityBindingElement _securityElement;
		private BinaryMessageEncodingBindingElement _binaryMessageEncodingElement;
		private HttpTransportBindingElement _httpTransportElement;

		private bool _useUsernameSecurity;

		public CustomHttpBinding()
		{
			_binaryMessageEncodingElement = new BinaryMessageEncodingBindingElement();
			_httpTransportElement = new HttpTransportBindingElement();
			SymmetricSecurityBindingElement element =
				SecurityBindingElement.CreateUserNameForSslBindingElement(true);
			_securityElement =
				SecurityBindingElement.CreateSecureConversationBindingElement(element, true);
			// property setting
			_useUsernameSecurity = false;
		}

		public CustomHttpBinding(string configName) : this()
		{
			CustomHttpBindingConfigElement element = CustomHttpBindingCollectionElement.GetBindingSection(configName);
			element.ApplyConfiguration(this);
		}

		public override BindingElementCollection CreateBindingElements()
		{
			BindingElementCollection elementCollection = new BindingElementCollection();

			if (_useUsernameSecurity) {
				elementCollection.Add(_securityElement);
			}
			elementCollection.Add(_binaryMessageEncodingElement);
			elementCollection.Add(_httpTransportElement);

			return elementCollection;
		}

		public override string Scheme
		{
			get { return _httpTransportElement.Scheme; }
		}

		public bool UsernameSecurityEnabled
		{
			get { return _useUsernameSecurity; }
			set { _useUsernameSecurity = value; }
		}

		public TransferMode TransferMode
		{
			get { return _httpTransportElement.TransferMode; }
			set { _httpTransportElement.TransferMode = value; }
		}
	}
}
