﻿using System;
using System.ServiceModel;
using System.Text;

namespace ServiceInheritanceClientApp
{
	// 수동으로 선언한 서비스의 베이스 인터페이스
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		int Order(string userid, string isbn, int amount);
		[OperationContract]
		bool Cancel(int orderid);
		[OperationContract]
		string[] Search(string title);
	}

	// 수동으로 작성한 베이스 인터페이스에 대한 프록시
	class BookStoreClient : ClientBase<IBookStore>, IBookStore
	{
		public BookStoreClient(EndpointAddress address, BasicHttpBinding binding)
			: base(binding, address)
		{
		}

		public int Order(string userid, string isbn, int amount)
		{
			return this.Channel.Order(userid, isbn, amount);
		}

		public bool Cancel(int orderid)
		{
			return this.Channel.Cancel(orderid);
		}

		public string[] Search(string title)
		{
			return this.Channel.Search(title);
		}
	}

	// 수동으로 작성한 서비스의 파생 인터페이스
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore2 : IBookStore
	{
		[OperationContract]
		string[] SearchEx(string title);
	}

	// 수동으로 작성한 파생 인터페이스에 대한 프록시
	class BookStore2Client : ClientBase<IBookStore2>, IBookStore2
	{
		public BookStore2Client(EndpointAddress address, BasicHttpBinding binding)
			: base(binding, address)
		{
		}

		//
		// 안타깝게도 베이스 인터페이스의 구현을 그대로 다시 작성해 주어야 한다.
		//
		public int Order(string userid, string isbn, int amount)
		{
			return this.Channel.Order(userid, isbn, amount);
		}

		public bool Cancel(int orderid)
		{
			return this.Channel.Cancel(orderid);
		}

		public string[] Search(string title)
		{
			return this.Channel.Search(title);
		}

		//
		// 파생 인터페이스에 대한 구현
		//
		public string[] SearchEx(string title)
		{
			return this.Channel.SearchEx(title);
		}
	}

	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("BookStore (Contract Inheritance) Client Application");

			int orderid;

			using (ServiceInheritanceClientApp.BookStore.BookStore2Client proxy =
						new ServiceInheritanceClientApp.BookStore.BookStore2Client()) {
				orderid = proxy.Order("GoPl", "999-9-999-99999-9", 2);
				Console.WriteLine("Place Order. Order ID = {0}", orderid);
				string[] results = proxy.SearchEx("WCF");
				Console.WriteLine("Search result :");
				foreach (string title in results) {
					Console.WriteLine("\t{0}", title);
				}
			}

			// 수동으로 작성한 프록시를 사용하여 서비스 호출
			EndpointAddress address = 
				new EndpointAddress("http://localhost/wcf/example/bookstoreservice");
			BasicHttpBinding binding = new BasicHttpBinding();

			using (BookStoreClient baseProxy = new BookStoreClient(address, binding)) {
				orderid = baseProxy.Order("BlackPearl", "000-0-000-00000-0", 2);
				Console.WriteLine("Place Order. Order ID = {0}", orderid);
			}

			using (BookStore2Client derivedProxy = new BookStore2Client(address, binding)) {
				derivedProxy.Cancel(orderid);
				string[] results = derivedProxy.SearchEx("WCF");
				Console.WriteLine("Search result :");
				foreach (string title in results) {
					Console.WriteLine("\t{0}", title);
				}
			}
		}
	}
}
