﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Collections;

namespace ServiceInheritanceServiceApp
{
	[ServiceContract(Namespace="http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		int Order(string userid, string isbn, int amount);
		[OperationContract]
		bool Cancel(int orderid);
		[OperationContract]
		string[] Search(string title);
	}

	[ServiceContract(Namespace="http://noname.com/bookstoreV2")]
	interface IBookStore2 : IBookStore
	{
		[OperationContract]
		string[] SearchEx(string title);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IBookStore
	{
		private static int id = 0;
		protected static string[] titles = new string[] {
			"WCF 10일만에 정복", "10일동안 하면 개똥이 만큼 WCF 한다", "WCF Programming",
			"Programming WCF", "Learning WCF", "WCF Step by Step" };
		protected static string[] authors = new string[] {
			"김정복", "김개똥", "김혜영",
			"김영철", "김철수", "김말똥" };

		// 도서 단가 반환
		// (고정 값... -_-; )
		private int GetUnitPrice(string isbn)
		{
			return 35000;
		}

		// 도서 주문
		public int Order(string userid, string isbn, int amount)
		{
			int total = GetUnitPrice(isbn) * amount;
			id += 1;
			// 주문 처리 로직
			// (단순 출력만... -_-; )
			Console.WriteLine(
				"Book Order: orderid={0}\n   userid={1}, isbn={2}, amount={3}, total={4}",
				id, userid, isbn, amount, total);
			// 주문 ID 반환
			return id;
		}

		// 주문 취소
		public bool Cancel(int orderid)
		{
			// 주문 취소 로직
			// (단순 출력만... ^^; )
			Console.WriteLine("Cancel Order: orderid={0}", orderid);
			return true;
		}

		// 도서명 검색
		public string[] Search(string title)
		{
			// 단순 검색 -_-;
			ArrayList results = new ArrayList();
			foreach (string t in titles) {
				if (t.StartsWith(title) == true) {
					results.Add(t);
				}
			}
			return (string[])results.ToArray(typeof(string));
		}
	}

	// 파생 인터페이스에 대한 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService2 : BookStoreService, IBookStore2
	{
		// 확장된 도서명 검색 -_-;
		public string[] SearchEx(string title)
		{
			ArrayList results = new ArrayList();
			foreach (string t in titles) {
				if (t.IndexOf(title) >= 0) {
					results.Add(t);
				}
			}
			return (string[])results.ToArray(typeof(string));
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Contract Inheritance Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService2),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore2),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
