﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.Xml;
using System.IO;
using System.Text;

namespace SoapActionService
{
	// 서비스 계약 정의
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract(
		   Action = "http://noname.com/bookstore/OrderAction",
		   ReplyAction = "http://noname.com/bookstore/OrderResponseAction")]
		int Order(string userid, string isbn, int amount);
		[OperationContract(
		   Action = "http://noname.com/bookstore/OrderAction2",
		   ReplyAction = "http://noname.com/bookstore/OrderResponseAction2")]
		int Order2(string userid, string isbn, int amount);
		[OperationContract(Action="*", ReplyAction="*")]
		Message ProcessGenericMessage(Message message);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IBookStore
	{
		private static int id = 0;

		private int GetUnitPrice(string isbn)
		{
			return 35000;
		}

		// 도서 주문
		public int Order(string userid, string isbn, int amount)
		{
			int total = GetUnitPrice(isbn) * amount;
			id += 1;
			// 주문 처리 로직
			// (단순 출력만... -_-; )
			Console.WriteLine(
				"Book Order: orderid={0}\n   userid={1}, isbn={2}, amount={3}, total={4}",
				id, userid, isbn, amount, total);
			// 주문 ID 반환
			return id;
		}

		// 도서 주문 2
		public int Order2(string userid, string isbn, int amount)
		{
			// 주문 처리 로직 2
			Console.WriteLine("Book Order2: Processing message...");
			return -1;
		}

		// 일반적인 메시지 처리
		public Message ProcessGenericMessage(Message message)
		{
			// Request 메시지를 표시한다.
			Console.WriteLine("Processing Generic Message:\n{0}", message.ToString());
			XmlDictionaryReader reader = message.GetReaderAtBodyContents();
			string elementName = reader.Name;
			string ns = reader.NamespaceURI;

			// Response 메시지에 포함될 Body를 '생성' 한다.
			MemoryStream stream = new MemoryStream();
			XmlWriter writer = XmlDictionaryWriter.CreateTextWriter(stream);
			writer.WriteStartElement(elementName + "Response", ns);
			writer.WriteEndElement();
			writer.Flush();

			// Response SOAP 메시지를 생성한다.
			stream.Position = 0;
			XmlDictionaryReader responseReader = XmlDictionaryReader.CreateTextReader(stream, XmlDictionaryReaderQuotas.Max);
			Message replyMessage = Message.CreateMessage(message.Version, "none", responseReader);
			replyMessage.Headers.RelatesTo = message.Headers.MessageId;

			// 반환될 메시지를 표시하고 결과 메시지를 반환한다.
			Console.WriteLine("Reply Message:\n{0}", replyMessage.ToString());
			return replyMessage;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("BookStore Service Example... (SOAP Action sample)");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			WSHttpBinding binding = new WSHttpBinding();
			binding.Security.Mode = SecurityMode.None;

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore),
				binding,
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
