﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;

namespace SimpleCustomBindingService
{
	// 서비스 계약 정의
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		int Order(string userid, string isbn, int amount);
		[OperationContract]
		bool Cancel(int orderid);
	}

	// 서비스 타입 구현
	class BookStoreService : IBookStore
	{
		private static int id = 0;

		private int GetUnitPrice(string isbn)
		{
			return 35000;
		}

		// 도서 주문
		public int Order(string userid, string isbn, int amount)
		{
			int total = GetUnitPrice(isbn) * amount;
			id += 1;
			// 주문 처리 로직
			// (단순 출력만... -_-; )
			Console.WriteLine(
				"Book Order: orderid={0}\n   userid={1}, isbn={2}, amount={3}, total={4}",
				id, userid, isbn, amount, total);
			// 주문 ID 반환
			return id;
		}

		// 주문 취소
		public bool Cancel(int orderid)
		{
			// 주문 취소 로직
			// (단순 출력만... ^^; )
			Console.WriteLine("Cancel Order: orderid={0}", orderid);
			return true;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Building a Binding Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			// BasicHttpBinding 의 기본 설정과 호환되는 커스텀 바인딩을 작성한다.
			// 이 코드는 다음 코드와 동등하다.
			//
			// BasicHttpBinding binding = new BasicHttpBinding();
			//
			//TextMessageEncodingBindingElement textMessageEncoding = new TextMessageEncodingBindingElement();
			//HttpTransportBindingElement httpTransport = new HttpTransportBindingElement();
			//CustomBinding binding = new CustomBinding();
			//// 바인딩 요소 추가 (순서가 중요하다)
			//binding.Elements.Add(textMessageEncoding);
			//binding.Elements.Add(httpTransport);
			//textMessageEncoding.MessageVersion = MessageVersion.Soap11;

			// 다음과 같이 configuration을 통해 CustomBinding을 구성할 수도 있다.
			CustomBinding binding = new CustomBinding("SimpleCustomBinding");

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IBookStore),
				binding,
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
