﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.IO;
using System.ServiceModel.Channels;

namespace FileMgrServiceApp
{
	// 서비스 계약 선언
	[ServiceContract]
	interface IFileMgr
	{
		[OperationContract]
		byte[] GetImageFile(string filename);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class FileMgrService : IFileMgr
	{
		public byte[] GetImageFile(string filename)
		{
			if (File.Exists(filename) == false) {
				throw new FileNotFoundException("Cannot find file......");
			}

			Console.WriteLine("Downloading image file, {0}", filename);
			FileStream stream = new FileStream(filename, FileMode.Open);
			byte[] buffer = new byte[stream.Length];
			stream.Read(buffer, 0, buffer.Length);
			stream.Close();
			return buffer;
		}
	}

	// 서비스 어플리케이션
	class ServiceApp
	{
		// 메인 메쏘드
		static void Main(string[] args)
		{
			Console.WriteLine("Transferring a File Example...");

			// 호스트 설정
			ServiceHost host = new ServiceHost(
				typeof(FileMgrService),
				new Uri("http://localhost/wcf/example/filemgrservice"),
				new Uri("net.tcp://localhost/wcf/example/filemgrservice"));

			// 바인딩 설정
			WSHttpBinding binding1 = new WSHttpBinding();
			binding1.Security.Mode = SecurityMode.None;

			WSHttpBinding binding2 = new WSHttpBinding();
			binding2.MessageEncoding = WSMessageEncoding.Mtom;
			binding2.Security.Mode = SecurityMode.None;

			NetTcpBinding binding3 = new NetTcpBinding();
			binding2.Security.Mode = SecurityMode.None;
			binding3.Security.Mode = SecurityMode.None;

			CustomBinding binding4 = new CustomBinding(
				new BinaryMessageEncodingBindingElement(),
				new HttpTransportBindingElement());

			// HTTP 기반 종점 추가
			host.AddServiceEndpoint(typeof(IFileMgr), binding1, "text");
			host.AddServiceEndpoint(typeof(IFileMgr), binding2, "mtom");
			host.AddServiceEndpoint(typeof(IFileMgr), binding3, "binary");
			host.AddServiceEndpoint(typeof(IFileMgr), binding4, "binary");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
