﻿using System;
using DataClientApp.DataService;
using System.Data;
using System.IO;
using System.IO.Compression;
using System.Runtime.Serialization.Formatters.Binary;

namespace DataClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Transfering a DataSet Sample Client......");

			int[] rowCounts = new int[] { 30, 100, 500, 1000, 3000, 5000, 10000, 20000 };

			GetDataSetTest(rowCounts);
			GetCompressedDataSetTest(rowCounts);
		}

		// 압축되지 않은 DataSet을 받는 예제 코드
		private static void GetDataSetTest(int[] rowCounts)
		{
			Console.WriteLine("Text Encoding Test...");
			foreach (int rows in rowCounts) {
				using (DataProviderClient proxy = new DataProviderClient("HttpTextEncoding")) {
					DataSet ds = proxy.GetData(rows);
					Console.WriteLine("\tDataSet Row Count : {0}", ds.Tables[0].Rows.Count);
				}
			}

			Console.WriteLine("MTOM Encoding Test...");
			foreach (int rows in rowCounts) {
				using (DataProviderClient proxy = new DataProviderClient("HttpMtomEncoding")) {
					DataSet ds = proxy.GetData(rows);
					Console.WriteLine("\tDataSet Row Count : {0}", ds.Tables[0].Rows.Count);
				}
			}

			Console.WriteLine("Binary Encoding Test...");
			foreach (int rows in rowCounts) {
				using (DataProviderClient proxy = new DataProviderClient("HttpBinaryEncoding")) {
					DataSet ds = proxy.GetData(rows);
					Console.WriteLine("\tDataSet Row Count : {0}", ds.Tables[0].Rows.Count);
				}
			}
		}

		// 바이트 배열에서 압축을 풀면서 DataSet으로 deserialize 한다.
		private static DataSet GetDataSetFromBytes(byte[] buffer)
		{
			MemoryStream stream = new MemoryStream(buffer);
			GZipStream gzipStream = new GZipStream(stream, CompressionMode.Decompress);
			BinaryFormatter formatter = new BinaryFormatter();
			DataSet ds = (DataSet)formatter.Deserialize(gzipStream);

			gzipStream.Close();
			stream.Close();
			return ds;
		}

		// 압축된 DataSet을 받는 예제 코드
		private static void GetCompressedDataSetTest(int[] rowCounts)
		{
			Console.WriteLine("Text Encoding with compression Test...");
			foreach (int rows in rowCounts) {
				using (DataProviderClient proxy = new DataProviderClient("HttpTextEncoding")) {
					byte[] result = proxy.GetDataEx(rows);
					DataSet ds = GetDataSetFromBytes(result);
					Console.WriteLine("\tDataSet Row Count : {0}", ds.Tables[0].Rows.Count);
				}
			}

			Console.WriteLine("MTOM Encoding with compression Test...");
			foreach (int rows in rowCounts) {
				using (DataProviderClient proxy = new DataProviderClient("HttpMtomEncoding")) {
					byte[] result = proxy.GetDataEx(rows);
					DataSet ds = GetDataSetFromBytes(result);
					Console.WriteLine("\tDataSet Row Count : {0}", ds.Tables[0].Rows.Count);
				}
			}

			Console.WriteLine("Binary Encoding with compression Test...");
			foreach (int rows in rowCounts) {
				using (DataProviderClient proxy = new DataProviderClient("HttpBinaryEncoding")) {
					byte[] result = proxy.GetDataEx(rows);
					DataSet ds = GetDataSetFromBytes(result);
					Console.WriteLine("\tDataSet Row Count : {0}", ds.Tables[0].Rows.Count);
				}
			}
		}
	}
}
