﻿using System;
using System.ServiceModel;
using System.ServiceModel.PeerResolvers;

namespace P2PChatApp
{
	// 서비스 계약 선언
	// P2P는 항상 One-way로만 메시지 전송을 해야 한다.
	[ServiceContract]
	interface IChat
	{
		[OperationContract(IsOneWay = true)]
		void Join(string nickname);
		[OperationContract(IsOneWay = true)]
		void SendMessage(string nickname, string msg);
		[OperationContract(IsOneWay = true)]
		void Leave(string nickname);
	}

	// 서비스 구현
	// 채팅 인터페이스의 구현으로 단순히 메시지를 출력한다.
	class ChatService : IChat
	{
		// 채팅에 참여 했음을 알림
		public void Join(string nickname)
		{
			Console.WriteLine("Peer joined: {0}", nickname);
		}
		// 채팅 메시지 전송
		public void SendMessage(string nickname, string msg)
		{
			Console.WriteLine("[{0}] {1}", nickname, msg);
		}
		// 채팅 종료를 알림
		public void Leave(string nickname)
		{
			Console.WriteLine("Peer Leaved: {0}", nickname);
		}
	}

	class ChatApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("P2P Chatting Sample App......");

			// 서비스 호스트 구성
			string meshAddress = "net.p2p://chatsample";
			ServiceHost host = new ServiceHost(typeof(ChatService));
			// 바인딩 구성 및 서비스 종점 생성
			NetPeerTcpBinding binding = new NetPeerTcpBinding();
			binding.Security.Mode = SecurityMode.None;
			host.AddServiceEndpoint(typeof(IChat), binding, meshAddress);
			// 서비스 오픈
			Console.WriteLine("Conversation Start......");
			host.Open();
			// 통신을 위한 클라이언트 프록시 구성
			EndpointAddress remoteAddress = new EndpointAddress(meshAddress);
			ChannelFactory<IChat> factory = new ChannelFactory<IChat>(binding, remoteAddress);
			IChat proxy = factory.CreateChannel();
			(proxy as IClientChannel).Open();
			// 채팅에 사용할 닉네임을 입력 받는다.
			string nickname;
			Console.Write("Your nickname: ");
			nickname = Console.ReadLine();
			// 채팅 서비스 호출
			Console.WriteLine("Input message...... input /quit to stop...");
			proxy.Join(nickname);
			while (true) {
				string msg = Console.ReadLine();
				if (String.Compare(msg, "/quit", true) == 0) {
					break;
				}
				proxy.SendMessage(nickname, msg);
			}
			proxy.Leave(nickname);
			(proxy as IDisposable).Dispose();
			// 서비스 호스트 종료
			host.Close();
			Console.WriteLine("Conversation end......");
		}
	}
}
