﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace TestServiceApp
{
	[ServiceContract]
	interface ITestContract
	{
		[OperationContract]
		void RaiseException();
	}

	class TestService : ITestContract
	{
		public void RaiseException()
		{
			// 테스트를 위해 예외를 발생시킨다
			throw new InvalidOperationException("Test Exception");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("ServiceHost Event Sample Service......");
			Console.WriteLine();

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/testservice"));
			host.AddServiceEndpoint(typeof(ITestContract), 
				new BasicHttpBinding(), "");
			// 이벤트 핸들러 설정
			host.Opening += new EventHandler(ReportServiceState);
			host.Opened += new EventHandler(ReportServiceState);
			host.Closing += new EventHandler(ReportServiceState);
			host.Closed += new EventHandler(ReportServiceState);
			host.Faulted += new EventHandler(ReportServiceState);
			// WSDL 활성화
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			host.Description.Behaviors.Add(behavior);
			behavior.HttpGetEnabled = true;
			// 서비스 시작
			Console.WriteLine("Before service open...");
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			// 서비스 종료
			host.Close();
		}

		static void ReportServiceState(object sender, EventArgs e)
		{
			ServiceHost host = (ServiceHost)sender;
			Console.WriteLine("Service state : {0}", host.State);
		}
	}
}
