﻿using System;
using System.ServiceModel;

using ExceptionClientApp.BadService;

namespace ExceptionClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("FaultException Handling Client Example......");

			using (BadContractClient proxy = new BadContractClient("WSHttpBinding_IBadContract")) {
				DoInvokeService(proxy);
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				DoInvokeService(proxy);
			}

			Console.WriteLine("-------------------------------------------------------------");

			// 서비스에서 FaultException<ExceptionDetail> 예외를 발생하므로 채널이 폴트 상태로
			// 바뀌는 상황은 발생하지 않는다.
			using (BadContractClient proxy = new BadContractClient("NetTcpBinding_IBadContract")) {
				DoInvokeService(proxy);
				Console.WriteLine("\nChannelState = {0}\n", proxy.InnerChannel.State);
				DoInvokeService(proxy);
			}
		}

		// 서비스 호출
		private static void DoInvokeService(BadContractClient proxy)
		{
			try {
				proxy.ThrowException();
			}
			catch (FaultException<ExceptionDetail> ex) {
				ExceptionDetail detail = ex.Detail;
				Console.WriteLine("Service FaultException [{0}] : {1}", detail.Type, detail.Message);
				Console.WriteLine("Stack Trace :\n{0}", detail.StackTrace);
			}
			catch (FaultException ex) {
				Console.WriteLine("Service Error::{0}", ex.Message);
			}
		}
	}
}
