﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace ExceptionServiceApp
{
	// 서비스 계약 선언
	[ServiceContract]
	interface IBadContract
	{
		[OperationContract]
		void ThrowException();
	}

	// 서비스 타입 구현
	[MyErrorHandler]
	class BadService : IBadContract
	{
		public void ThrowException()
		{
			throw new InvalidOperationException("Invalid method invocation");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Error Handler Sample Service......");

			// 사용자 정의 ServiceHost 사용
			//UseCustomServiceHost();

			// ServiceBehavior 사용
			// 주) 서비스 타입에 명시된 MyErrorHandler 특성을 제거하지 않으면 에러 핸들러가
			//     2회 호출된다.
			//UseServiceBehavior();
			
			// 사용자 정의 Attribute 사용
			UseCustomAttribute();
		}

		// 사용자 정의 ServiceHost를 이용하여 에러 핸들러 사용
		static void UseCustomServiceHost()
		{
			// 베이스 주소 및 종점 설정은 configuration 파일을 사용한다.
			MyServiceHost host = new MyServiceHost(typeof(BadService));
			// 서비스 구동
			host.Open();
			Console.WriteLine("Press any key to stop the service");
			Console.ReadKey(true);
			host.Close();
		}

		// Service Behavior를 이용하여 에러 핸들러 사용
		private static void UseServiceBehavior()
		{
			ServiceHost host = new ServiceHost(typeof(BadService));
			// 에러 핸들러 설치를 위한 Behavior 설정
			MyErrorHandlerBehavior behavior =
				host.Description.Behaviors.Find<MyErrorHandlerBehavior>();
			if (behavior == null) {
				behavior = new MyErrorHandlerBehavior();
				host.Description.Behaviors.Add(behavior);
			}
			behavior.Enabled = true;
			// 서비스 구동
			host.Open();
			Console.WriteLine("Press any key to stop the service");
			Console.ReadKey(true);
			host.Close();
		}

		// 사용자 정의 특성을 이용하여 에러 핸들러 사용
		private static void UseCustomAttribute()
		{
			// 베이스 주소 및 종점 설정은 configuration 파일을 사용한다.
			// 일반적인 ServiceHost 클래스를 사용하고 있음에 유의한다.
			ServiceHost host = new ServiceHost(typeof(BadService));
			// 서비스 구동
			host.Open();
			Console.WriteLine("Press any key to stop the service");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
