﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace ExceptionServiceApp
{
	[ServiceContract]
	interface IBadContract
	{
		[OperationContract]
		void ThrowSimpleFaultException();
		[OperationContract]
		void ThrowWrappedFaultException();
		[OperationContract]
		void ThrowFaultExceptionOfT();
		[OperationContract]
		void ThrowFaultExceptionOfExceptionDetail();
	}

	class BadService : IBadContract
	{
		public void ThrowSimpleFaultException()
		{
			Console.WriteLine("Throwing simple FaultException");
			throw new FaultException("Test FaultException");
		}

		public void ThrowWrappedFaultException()
		{
			Console.WriteLine("Throwing Wrapped FaultException");
			try {
				DoThrow();
			}
			catch (Exception ex) {
				// 발생한 예외를 FaultException으로 wrapping 한다.
				FaultCode code = FaultCode.CreateReceiverFaultCode(
					new FaultCode("Server." + ex.GetType().Name));
				FaultReason reason = new FaultReason(ex.Message);
				MessageFault messageFault = MessageFault.CreateFault(code, reason, ex);
				FaultException faultException = FaultException.CreateFault(messageFault, ex.GetType());

				throw faultException;
			}
		}

		public void ThrowFaultExceptionOfT()
		{
			Console.WriteLine("Throwing FaultException<T>");
			try {
				DoThrow();
			}
			catch (NullReferenceException ex) {
				throw GetFaultExeption<NullReferenceException>(ex, "Exception Test");
			}
			catch (Exception ex) {
				throw GetFaultExeption<Exception>(ex);
			}
		}

		// 발생한 예외를 FaultException<NullReferenceException>으로 wrapping 한다.
		private FaultException GetFaultExeption<T>(T exception) where T : Exception
		{
			return GetFaultExeption<T>(exception, exception.Message);
		}

		// 발생한 예외를 FaultException<NullReferenceException>으로 wrapping 한다.
		private FaultException GetFaultExeption<T>(T exception, string reason) where T : Exception
		{
			FaultCode code = FaultCode.CreateReceiverFaultCode(new FaultCode(exception.GetType().Name));
			FaultException<T> faultException =
				new FaultException<T>(exception, reason, code);
			return faultException;
		}

		public void ThrowFaultExceptionOfExceptionDetail()
		{
			Console.WriteLine("Throwing FaultException<ExceptionDetail>");
			try {
				DoThrow();
			}
			catch (Exception ex) {
				// 발생한 예외를 FaultException<ExceptionDetail>으로 wrapping 한다.
				FaultCode code = FaultCode.CreateReceiverFaultCode(new FaultCode(ex.GetType().Name));
				ExceptionDetail detail = new ExceptionDetail(ex);
				FaultException<ExceptionDetail> faultException =
					new FaultException<ExceptionDetail>(detail, ex.Message, code);
				throw faultException;
			}
		}

		private void DoThrow()
		{
			object obj = null;
			// null reference exception 발생
			obj.GetType();
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("FaultException Sample Service......");

			// 베이스 주소 및 종점 설정은 configuration 파일을 사용한다.
			ServiceHost host = new ServiceHost(typeof(BadService));

			host.Open();
			Console.WriteLine("Press any key to stop the service");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
