﻿using System;
using System.ServiceModel;

namespace PerSessionServiceApp
{
	// 서비스 계약 선언
	[ServiceContract]
	interface IPerSession
	{
		[OperationContract]
		int TestMethod();
	}

	// PerSession 서비스 타입 구현
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession)]
	class PerSessionService : IPerSession, IDisposable
	{
		private int _CallCount;

		public PerSessionService()
		{
			Console.WriteLine(".ctor()   : Id={0}, Session={1}",
				this.GetHashCode(), OperationContext.Current.SessionId);
			_CallCount = 0;
		}

		public void Dispose()
		{
			Console.Write("Dispose() : Id={0}, ", this.GetHashCode());
			if (OperationContext.Current == null) {
				Console.WriteLine("OperationContext.Current == null");
			}
			else {
				Console.WriteLine("Session={1}",
					this.GetHashCode(), OperationContext.Current.SessionId);
			}
		}

		public int TestMethod()
		{
			_CallCount++;
			Console.WriteLine("*** TestMethod() : CallCount={0}", _CallCount);
			return _CallCount;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("PerSession Service Test Sample......");

			// 서비스 호스트와 종점 설정은 configuration을 사용한다.
			ServiceHost host = new ServiceHost(typeof(PerSessionService));
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
