﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Diagnostics;
using TestClientApp.PerCall;
using TestClientApp.PerSession;
using TestClientApp.Singleton;

namespace TestClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Throttling Sample Client......");

			// 세션을 사용하지 않는 바인딩 사용
			//TestPerCallService("BasicHttpBinding_IPerCallService");
			// 세션을 사용하는 바인딩 사용
			//TestPerCallService("NetTcpBinding_IPerCallService");

			// 세션을 사용하지 않는 바인딩 사용
			//TestPerSessionService("BasicHttpBinding_IPerSessionService");
			// 세션을 사용하는 바인딩 사용
			TestPerSessionService("NetTcpBinding_IPerSessionService");

			// 세션을 사용하지 않는 바인딩 사용
			//TestSingletonService("BasicHttpBinding_ISingletonService");
			// 세션을 사용하는 바인딩 사용
			//TestSingletonService("NetTcpBinding_ISingletonService");
		}

		private static void TestPerCallService(string endpointName)
		{
			ThreadStart func = delegate() {
				using (PerCallServiceClient proxy = new PerCallServiceClient(endpointName)) {
					Stopwatch clock = new Stopwatch();
					clock.Start();
					proxy.TestMethod(1000);
					clock.Stop();
					Console.WriteLine("[{0}] End of service call. time = {1} msec",
						Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);

				}
			};

			Console.WriteLine("PerCallService Test with endpoint, '{0}'", endpointName); 
			Thread[] threads = new Thread[10];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
			}
			// 쓰레드들을 시작한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
		}

		private static void TestPerSessionService(string endpointName)
		{
			ThreadStart func = delegate() {
				using (PerSessionServiceClient proxy = new PerSessionServiceClient(endpointName)) {
					Stopwatch clock = new Stopwatch();
					clock.Start();
					proxy.TestMethod(1000);
					clock.Stop();
					Console.WriteLine("[{0}] End of service call. time = {1} msec",
						Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);

				}
			};

			Console.WriteLine("PerSessionService Test with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[10];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
			}
			// 쓰레드들을 시작한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
		}

		private static void TestSingletonService(string endpointName)
		{
			ThreadStart func = delegate() {
				using (SingletonServiceClient proxy = new SingletonServiceClient(endpointName)) {
					Stopwatch clock = new Stopwatch();
					clock.Start();
					proxy.TestMethod(1000);
					clock.Stop();
					Console.WriteLine("[{0}] End of service call. time = {1} msec",
						Thread.CurrentThread.ManagedThreadId, clock.ElapsedMilliseconds);

				}
			};

			Console.WriteLine("TestSingletonService Test with endpoint, '{0}'", endpointName);
			Thread[] threads = new Thread[10];
			// 쓰레드를 생성한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i] = new Thread(func);
			}
			// 쓰레드들을 시작한다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Start();
			}
			// 쓰레드 종료를 기다린다.
			for (int i = 0; i < threads.Length; i++) {
				threads[i].Join();
			}
		}
	}
}
