﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Timers;

namespace TimeServiceApp
{
	[ServiceContract]
	interface ITimeCallback
	{
		[OperationContract]
		void OnTick(DateTime serverTime);
	}

	[ServiceContract(CallbackContract = typeof(ITimeCallback))]
	interface ITime
	{
		[OperationContract]
		void StartTimeService();
	}

	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single,
		ConcurrencyMode = ConcurrencyMode.Single)]
	class TimeService : ITime, IDisposable
	{
		private static List<ITimeCallback> _Clients = new List<ITimeCallback>();
		private static Timer _Timer;

		public TimeService()
		{
			_Timer = new Timer(1000);
			_Timer.Elapsed += delegate(object sender, ElapsedEventArgs e) {
				List<ITimeCallback> errorList = new List<ITimeCallback>();
				DateTime now = DateTime.Now;
				foreach (ITimeCallback callback in _Clients) {
					IClientChannel channel = callback as IClientChannel;
					if (channel.State == CommunicationState.Opened) {
						callback.OnTick(now);
					}
					else {
						// 에러 로깅
						Console.WriteLine("Session Closed : {0}", channel.SessionId);
						// 오류 목록에 추가
						errorList.Add(callback);
					}
					//try {
					//    callback.OnTick(now);
					//}
					//catch (ObjectDisposedException) {
					//    // 에러 로깅
					//    Console.WriteLine("Callback error : callback proxy is disposed");
					//    // 오류 목록에 추가
					//    errorList.Add(callback);
					//}
				}
				// 오류를 유발한 콜백을 목록에서 제거한다.
				foreach (ITimeCallback callback in errorList) {
					_Clients.Remove(callback);
				}
			};
			_Timer.Enabled = true;
		}

		void IDisposable.Dispose()
		{
			_Timer.Enabled = false;
			foreach (ITimeCallback callback in _Clients) {
				(callback as IDisposable).Dispose();
			}
		}

		public void StartTimeService()
		{
			// Singleton 인스턴스 모드와 Single 병행성 모드는 쓰레드 동기화
			// 코르를 사용하지 않고 공유되는 리스트 업데이트가 가능하도록 해준다.
			OperationContext ctx = OperationContext.Current;
			ITimeCallback callback = ctx.GetCallbackChannel<ITimeCallback>();
			_Clients.Add(callback);
		}

	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Server Time Service Sample......");

			NetTcpBinding netTcpBinding = new NetTcpBinding();
			netTcpBinding.Security.Mode = SecurityMode.None;
			netTcpBinding.ReliableSession.Enabled = false;

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;

			ServiceHost host = new ServiceHost(
				typeof(TimeService),
				new Uri("http://localhost/wcf/example/callback/time"),	// for metadata only
				new Uri("net.tcp://localhost/wcf/example/callback/time"));
			host.AddServiceEndpoint(typeof(ITime), netTcpBinding, "");
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Time service started. Press Any Key to Stop......");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
